#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <AppKit.h>
#include <InterfaceKit.h>
#include <MediaDefs.h>
#include <Path.h>
#include "libmpeg3.h"
#include "SoundPlayAc3decPlugin.h"

/* ===========================================================
	Plugin-descriptor structure.
	This structure is used to gather information about a plugin
	and	instantiate and destroy specific instances of a plugin.
*/
static plugin_descriptor plugin={
	PLUGIN_DESCRIPTOR_MAGIC,PLUGIN_DESCRIPTOR_VERSION,
	"ac3dec",1, PLUGIN_IS_DECODER,

	"Ac3dec decoder",
	"By Jerome Duval\n"
	"\n"
	"Based on my Libmpeg3 port for BeOS\n"
	"\n",
	&aboutplugin,
	NULL, // configure
	&getplugin,
	&destroyplugin
};

/* ===========================================================
	Plugin-operations structure.
	This is used to actually perform operations on a plugin.
	It is returned by the getplugin() function from the 
	plugin_descriptor structure above.
*/
static decoder_plugin_ops ops={
	PLUGIN_DECODER_MAGIC,PLUGIN_DECODER_VERSION,

	NULL,	//open,	// open
	NULL,	//close,	// close
	info,	//info,
	readdata,
	NULL,	// play
	NULL,	// stop
	NULL,	// setvolume
	NULL,	// setpitchandspeed
	seek,
	position,
	NULL  // bufferamount
};


/* ===========================================================
	Return the list of plugins in this file.
*/
static plugin_descriptor *plugs[]={
	&plugin,
	0
};

plugin_descriptor **get_plugin_list(void)
{
	return plugs;
}

/* ===========================================================
	Return the list of filetypes supported by all the plugins
	in this file
*/
static supported_type supportedtypes[]=
{
	{"audio/x-ac3",		"Ac3 file",							"Ac3 file",	"ac3", NULL },
	{NULL,				NULL,															NULL,						NULL,NULL}
};

supported_type *get_supported_types(void)
{
	return supportedtypes;
}


/* ===========================================================
	Implementation of the plugin_descriptor functions
*/
//static bool mmRegistered=false;
static void *getplugin(void **data, const char *name, const char *header, uint32 size, plugin_info *info)
{
	if(info)
	{
		// Loading a new instance of this plugin is an easy way
		// around rewriting everything to use local variables.
		// Using the "main" image as well instead of always
		// loading a new image is left as an excercise for the reader.
		BEntry entry(info->ref);
		if(!entry.Exists())
			return NULL;
		BPath path(&entry);
		image_id img=load_add_on(path.Path());
		if(img<0)
			return NULL;
		plugin_descriptor** (*declistfunc)(void);
		if(B_OK!=get_image_symbol(img, "get_plugin_list", B_SYMBOL_TYPE_TEXT, (void**)&declistfunc))
			return NULL;
		plugin_descriptor *plugdesc=*(declistfunc());
		void *imgops=plugdesc->Instantiate_Plugin(data,name,header,size,NULL);
		if(imgops)
		{
			file_handle *fh=(file_handle*)*data;
			fh->img=img;
			fh->plugdesc=plugdesc;
			return imgops;
		}
		unload_add_on(img);
	}
	else
	{
		file_handle *fh=new file_handle;
		if(mpeg3_check_sig((char*)name)==1)
			{
				fh->fd = mpeg3_open((char*)name);
				if(mpeg3_has_audio(fh->fd)) {
					*data=(void*)fh;
					return &ops;
				}
			}
		delete fh;
	}
	return NULL;
}

static void destroyplugin(void *ops,void *_fh)
{
	file_handle *fh=(file_handle*)_fh;
	if(ops)
	{
		mpeg3_close(fh->fd);
		fh->plugdesc->Destroy_Plugin(NULL,_fh);
		unload_add_on(fh->img);
		delete fh;
		return;
	}
}

// Normally you wouldn't implement the about function for such a simple
// message. It is done here for completeness.
static void aboutplugin()
{
	BAlert *alert=new BAlert("About Ac3dec decoder",
		"Ac3dec decoder\n"
		"\n"
		"It handles file types supported by Libmpeg3\n"
		"(mpg, ac3, mp3, mp2)\n\n"
		"\n",
		"OK");
	alert->SetFeel(B_NORMAL_WINDOW_FEEL);
	alert->Go(NULL);
}


/* ===========================================================
	Implementation of the decoder_plugin_ops functions
*/
int32 readdata(void *_mh, char *buf, ulong count)
{
	file_handle *fh=(file_handle*)_mh;
	
	mpeg3_read_audio(fh->fd, NULL, (short*)buf, fh->nbchannels, count, 0);
		
	return count;
}

status_t info(void *_mh, file_info *info)
{
	char str[30];
	file_handle *fh=(file_handle*)_mh;
	
	// Fill in the file_info structure
	strcpy(info->name,"");
	
	info->bitrate=fh->fd->atrack[0]->audio->bitrate;
	
	switch(fh->fd->atrack[0]->audio->format)
		{
			case AUDIO_AC3:
				strcpy(info->mimetype,"audio/x-ac3");
				strcpy(info->typedesc,"ac3");
				sprintf(str, ", %-3.0fkb", info->bitrate);
				strcat(info->typedesc, str);
				break;	
				
			case AUDIO_MPEG:
				switch(fh->fd->atrack[0]->audio->layer)
				{
					case 1:
						break;

					case 2:
						strcpy(info->mimetype,"audio/x-mpeg2");
						strcpy(info->typedesc,"mpeg layer 2");
						
						break;

					case 3:
						strcpy(info->mimetype,"audio/x-mpeg3");
						strcpy(info->typedesc, "mpeg layer 3");
						break;

					default:
						break;
				}
				sprintf(str, ", %ikb", (int)(info->bitrate)/1000);
				strcat(info->typedesc, str);
				break;
			
			case AUDIO_PCM:
				strcpy(info->mimetype,"audio/pcm");
				strcpy(info->typedesc,"pcm");
				break;
		}
	
	info->samplerate=mpeg3_sample_rate(fh->fd, 0);
	
	info->numchannels=mpeg3_audio_channels(fh->fd, 0);
	sprintf(str, ", %i channel", (int)info->numchannels);
	if(info->numchannels>1)	strcat(str, "s");
	strcat(info->typedesc, str);
	if(info->numchannels>2) {
		info->numchannels = 2;
		fh->nbchannels = -2;
	} else
		fh->nbchannels = -1;
	
	info->granularity=1;
	info->framecount=mpeg3_audio_samples(fh->fd, 0);
	info->samplesize=2;
	if(B_HOST_IS_LENDIAN)
		info->byteorder=B_LITTLE_ENDIAN;
	else
		info->byteorder=B_BIG_ENDIAN;
	info->sampleformat=B_LINEAR_SAMPLES;
	info->flags=0;
	
	return B_OK;
}

static status_t	seek(void *_mh, uint32 pos)
{
	file_handle *fh=(file_handle*)_mh;
	mpeg3_set_sample(fh->fd, pos, 0);

	return B_OK;
}

static uint32 position(void *_mh)
{
	file_handle *fh=(file_handle*)_mh;
	uint32 pos = mpeg3_get_sample(fh->fd, 0);
	
	return pos;
}

int main()
{
	BApplication app("application/x-ac3plugin");
	BAlert *alert=new BAlert("About Ac3dec decoder",
		"Ac3dec decoder\n"
		"\n"
		"It handles file types supported by Libmpeg3\n"
		"(mpg, ac3, mp3, mp2)\n\n"
		"\n",
		"OK");
	alert->SetFeel(B_NORMAL_WINDOW_FEEL);
	alert->Go();
}


